$ifndef _SPI33_INC_
$define _SPI33_INC_
'
' SPIxCON Register
'
$define cSCK_PIN_OFF     $FFFF ' Internal SPI clock is disabled, pin functions as I/O
$define cSCK_PIN_ON      $EFFF ' Internal SPI clock is enabled

$define cSDO_PIN_OFF     $FFFF ' SDO pin is not used by module
$define cSDO_PIN_ON      $F7FF ' SDO pin is  used by module

$define cSPI_MODE16      $FFFF ' Communication is word wide
$define cSPI_MODE8       $FBFF ' Communication is byte wide

$define cSPI_SMP_ON      $FFFF ' Input data sampled at end of data output time
$define cSPI_SMP_OFF     $FDFF ' Input data sampled at middle of data output time

$define cSPI_CKE_ON      $FFFF ' Transmit happens from active clock state to idle clock state
$define cSPI_CKE_OFF     $FEFF ' Transmit happens on transition from idle clock state to active clock state

$define cSPI_SLAVE_ON    $FFFF ' Slave Select enable
$define cSPI_SLAVE_OFF   $FF7F ' Slave Select not used by module

$define cCLK_ACTIVE_LOW  $FFFF ' Idle state for clock is high, active is low
$define cCLK_ACTIVE_HIGH $FFBF ' Idle state for clock is low, active is high

$define cSPI_MASTER_ON   $FFFF ' Master Mode
$define cSPI_MASTER_OFF  $FFDF ' Slave Mode

$define cSEC_PRE_1_1     $FFFF ' Secondary Prescale 1:1
$define cSEC_PRE_2_1     $FFFB ' Secondary Prescale 2:1
$define cSEC_PRE_3_1     $FFF7 ' Secondary Prescale 3:1
$define cSEC_PRE_4_1     $FFF3 ' Secondary Prescale 4:1
$define cSEC_PRE_5_1     $FFEF ' Secondary Prescale 5:1
$define cSEC_PRE_6_1     $FFEB ' Secondary Prescale 6:1
$define cSEC_PRE_7_1     $FFE7 ' Secondary Prescale 7:1
$define cSEC_PRE_8_1     $FFE3 ' Secondary Prescale 8:1

$define cPRI_PRE_1_1     $FFFF ' Primary Prescale 1:1
$define cPRI_PRE_4_1     $FFFE ' Primary Prescale 4:1
$define cPRI_PRE_16_1    $FFFD ' Primary Prescale 16:1
$define cPRI_PRE_64_1    $FFFC ' Primary Prescale 64:1
'
' Common defines for SPI master modes with SPI1CON
'
$define cSPI_MODE0 cSCK_PIN_ON & cSDO_PIN_ON & cSPI_MODE8 & cSPI_SMP_ON & cSPI_CKE_ON & cCLK_ACTIVE_HIGH & cSPI_MASTER_ON
'
'SPIXCON2 Register
'
$define cFRAME_ENABLE_ON    $FFFF ' Frame SPI support enable
$define cFRAME_ENABLE_OFF   $7FFF ' Frame SPI support Disable

$define cFRAME_SYNC_INPUT   $FFFF ' Frame sync pulse Input (slave)
$define cFRAME_SYNC_OUTPUT  $BFFF ' Frame sync pulse Output (master)

$define FRAME_ACTIVE_HIGH   $FFFF ' Frame sync pulse is active-high
$define cFRAME_ACTIVE_LOW   $DFFF ' Frame sync pulse is active-low

$define cFRAME_SYNC_EDGE_COINCIDE $FFFF ' Frame sync pulse coincides with first bit clock
$define cFRAME_SYNC_EDGE_PRECEDE  $FFFD ' Frame sync pulse precedes first bit clock

$define cFIFO_BUFFER_ENABLE  $FFFF ' FIFO buffer enabled
$define cFIFO_BUFFER_DISABLE $FFFE ' FIFO buffer enabled
'
' SPIxSTAT Register
'
$define cSPI_ON             $FFFF ' Enable module
$define cSPI_OFF            $7FFF ' Disable module

$define cSPI_IDLE_ON        $DFFF ' Continue module operation in idle mode
$define cSPI_IDLE_OFF       $FFFF ' Discontinue module operation in idle mode

$define cSPI_RX_OVFLOW_CLR  $FFBF ' Clear receive overflow bit.

$define cFIFO_LEN_1         $FFF8 ' FIFO buffer length 1 words
$define cFIFO_LEN_2         $FFF9 ' FIFO buffer length 2 words
$define cFIFO_LEN_3         $FFFA ' FIFO buffer length 3 words
$define cFIFO_LEN_4         $FFFB ' FIFO buffer length 4 words
$define cFIFO_LEN_5         $FFFC ' FIFO buffer length 5 words
$define cFIFO_LEN_6         $FFFD ' FIFO buffer length 6 words
$define cFIFO_LEN_7         $FFFE ' FIFO buffer length 7 words
$define cFIFO_LEN_8         $FFFF ' FIFO buffer length 8 words

$define cSPI_INT_EN         $FFFF ' SPI Interrupt Enable
$define cSPI_INT_DIS        $FFF7 ' SPI Interrupt Disable

$define cSPI_INT_PRI_0      $FFF8 ' SPI Interrupt Prior Level_0
$define cSPI_INT_PRI_1      $FFF9 ' SPI Interrupt Prior Level_1
$define cSPI_INT_PRI_2      $FFFA ' SPI Interrupt Prior Level_2
$define cSPI_INT_PRI_3      $FFFB ' SPI Interrupt Prior Level_3
$define cSPI_INT_PRI_4      $FFFC ' SPI Interrupt Prior Level_4
$define cSPI_INT_PRI_5      $FFFD ' SPI Interrupt Prior Level_5
$define cSPI_INT_PRI_6      $FFFE ' SPI Interrupt Prior Level_6
$define cSPI_INT_PRI_7      $FFFF ' SPI Interrupt Prior Level_7

'-------------------------------------------------------------------------------
' ** SPI1 peripheral **
'
$ifdef _SPI1CON1
'----------------------------------------------------------------------------
' Open_SPI1
' Input     : pConfig1 - This contains the parameters to be configured in the SPIxCON1 register as defined below:
'             SCK Pin Control bit
'               cSCK_PIN_OFF
'               cSCK_PIN_ON
'             SDO Pin Control bit
'               cSDO_PIN_OFF
'               cSDO_PIN_ON
'             16-bit/8-bit Communication mode
'               cSPI_MODE16
'               cSPI_MODE8
'             SPI Data Input Sample phase
'               cSPI_SMP_ON
'               cSPI_SMP_OFF
'             SPI Clock Edge Select
'               cSPI_CKE_ON
'               cSPI_CKE_OFF
'             SPI slave select enable
'               cSPI_SLAVE_ON
'               cSPI_SLAVE_OFF
'             SPI Clock polarity select
'               cCLK_ACTIVE_LOW
'               cCLK_ACTIVE_HIGH
'             SPI Mode Select bit
'               cSPI_MASTER_ON
'               cSPI_MASTER_OFF
'             Secondary Prescale select
'               cSEC_PRE_1_1
'               cSEC_PRE_2_1
'               cSEC_PRE_3_1
'               cSEC_PRE_4_1
'               cSEC_PRE_5_1
'               cSEC_PRE_6_1
'               cSEC_PRE_7_1
'               cSEC_PRE_8_1
'             Primary Prescale select
'               cPRI_PRE_1_1
'               cPRI_PRE_4_1
'               cPRI_PRE_16_1
'               cPRI_PRE_64_1
'             Final SPI speed (in KHz) = ((fOSC / 2) / 1000) / (Secondary Prescale * Primary Prescale)
'
'           : pConfig2 - This contains the parameters to be configured in the SPIxSTAT register as defined below:
'             SPI Enable/Disable
'               cSPI_ON
'               cSPI_OFF
'             SPI Idle mode operation
'               cSPI_IDLE_ON
'               cSPI_IDLE_OFF
'             Clear Receive Overflow Flag bit
'               cSPI_RX_OVFLOW_CLR
' Output    : None
' Notes     : This macros initialises the SPI1 module and sets the Idle mode Operation.
'           : It clears SPI1CON2 to disable Frame operations
'
$define Open_SPI1(pConfig1,pConfig2) '
    SPI1CON1 = pConfig1             '
    SPI1CON2 = 0                    '
    SPI1STAT = pConfig2

'----------------------------------------------------------------------------
' ConfigInt_SPI1
' This macro configures the SPI Interrupt
' Input     : pConfig - SPI interrupt priority and enable/disable information as defined below
'           : Interrupt enable/disable
'               cSPI_INT_EN
'               cSPI_INT_DIS
'           : Interrupt Priority
'               cSPI_INT_PRI_0
'               cSPI_INT_PRI_1
'               cSPI_INT_PRI_2
'               cSPI_INT_PRI_3
'               cSPI_INT_PRI_4
'               cSPI_INT_PRI_5
'               cSPI_INT_PRI_6
'               cSPI_INT_PRI_7
' Output    : None
' Notes     : This macro clears the Interrupt Flag bit, sets the interrupt priority
'           : and enables/disables the interrupt
'
$define ConfigInt_SPI1(pConfig) '
    IFS0bits_SPI1IF = 0        '
    IPC2bits_SPI1IP0 = pConfig & $07 '
    IEC0bits_SPI1IE = (pConfig & $08) >> 3

'----------------------------------------------------------------------------
' Close_SPI1
' This macro turns off the SPI module
' Input     : None
' Output    : None
' Notes     : This macro disables the SPI interrupt and then turns off the module.
'           : The Interrupt Flag bit is also cleared
'
$define Close_SPI1()       '
    IEC0bits_SPI1IE = 0    '
    SPI1STATbits_SPIEN = 0 '
    IFS0bits_SPI1IF = 0

'----------------------------------------------------------------------------
' Read_SPI1
' This procedure reads the content of the SPI Receive Buffer (SPIxBUF) register.
' Input     : None
' Output    : This procedure returns the content of Receive Buffer (SPIxBUF) register.
' Notes     : This procedure returns the content of the Receive Buffer register.
'           : If 16-bit communication is enabled, the data in the SPIxBUF register is returned.
'             If 8-bit communication is enabled, then the lower byte of SPIxBUF is returned.
'
Proc Read_SPI1(), WREG0.Byte0
    SPI1BUF = $00FF
    While SPI1STATbits_SPITBF = 1 : Wend  
    While SPI1STATbits_SPIRBF = 0 : Wend
    Result =  SPI1BUF
EndProc
'----------------------------------------------------------------------------
' Write_SPI1
' This macro writes the data to be transmitted into the Transmit Buffer (SPIxBUF) register.
' Input     : pDataOut - This is the data to be transmitted which will be stored in SPI buffer.
' Output    : None
' Notes     : This macro writes the data (byte/word) to be transmitted into the transmit buffer.
'           : If 16-bit communication is enabled, the 16-bit value is written to the transmit buffer.
'           : If 8-bit communication is enabled, then upper byte is masked and then written to the transmit buffer.
'
Proc Write_SPI1(pDataOut As SPI1BUF.Byte0)
    While SPI1STATBits_SPITBF = 1 : Wend  
    While SPI1STATBits_SPIRBF = 0 : Wend
    WREG0.Byte0 = SPI1BUF
EndProc
'    SPI1BUF = pDataOut       '
'    While SPI1STATbits_SPITBF = 1 : Wend
    
'----------------------------------------------------------------------------
' DataRdy_SPI1
' This macro determines if the SPI buffer contains any data to be read.
' Input     : None
' Output    : If 1 is returned, it indicates that the data has been
'             received in the receive buffer and is to be read.
'             If 0 is returned, it indicates that the receive is not
'             complete and the receive buffer is empty.
' Notes     : This macro returns the status of SPI receive buffer.
'             This indicates if the SPI receive buffer contains any new
'             data that is yet to be read as indicated by the
'             SPIxSTAT<SPIRBF> bit. This bit is cleared by hardware
'             when the data is read from the buffer.
'
$define DataRdy_SPI1() SPI1STATbits_SPIRBF

'----------------------------------------------------------------------------
' EnableInt_SPI1
' Enables SPIx Transfer Complete Interrupt
' Input     : None
' Notes     : This macro sets SPI Interrupt Enable bit of Interrupt Enable Control register.
'
$define EnableInt_SPI1() IEC0bits_SPI1IE = 1

'----------------------------------------------------------------------------
' DisableInt_SPI1
' This macro disables the SPI Interrupt
' Input     : None
' Notes     : This macro clears SPI Interrupt Enable bit of Interrupt Enable Control register.
'
$define DisableInt_SPI1() IEC0bits_SPI1IE = 0

'----------------------------------------------------------------------------
' SetPriorityInt_SPI1(priority)
' This macro sets the priority level for SPI interrupt
' Input     : Priority - This input parameter is the level of interrupt priority
' Notes     : None
'
$define SetPriorityInt_SPI1(pPriority) IPC2bits_SPI1IP0 = pPriority

'----------------------------------------------------------------------------
' SPI1_Clear_Intr_Status_Bit
' Macro to Clear SPI Interrupt Status bit
' Input     : None
' Notes     : None
'
$define SPI1_Clear_Intr_Status_Bit() IFS0bits_SPI1IF = 0

'----------------------------------------------------------------------------
' SPI1_Clear_Recv_OV
' macro clears Receive Overflow flag
' Input     : None
' Notes     : None
'
$define SPI1_Clear_Recv_OV() SPI1STATbits_SPIROV = 0

'----------------------------------------------------------------------------
' SPI1_Tx_Buf_Full
' macro reads the status of Transmit Buffer.
' Input     : None
' Output    : 1 - Indicates buffer is full , Transmit is not yet stated
'           : 0 - Indicates buffer is empty , Transmit is complete
' Notes     : None
'
$define SPI1_Tx_Buf_Full() SPI1STATbits_SPITBF

'----------------------------------------------------------------------------
' SPI1_Rx_Buf_Full
' macro reads the status of Receive Buffer.
' Input     : None
' Outout    : 1 - Indicates Receive complete
'           : 0 - Indicates not yet received
' Notes     : None
'
$define SPI1_Rx_Buf_Full() SPI1STATbits_SPIRBF

$define SPI1_Open(pConfig1,pConfig2) Open_SPI1(pConfig1,pConfig2)
$define SPI1_ConfigInt(pConfig) ConfigInt_SPI1(pConfig)
$define SPI1_Close() Close_SPI1()
$define SPI1_Read() Read_SPI1()
$define SPI1_Write(pDataOut) Write_SPI1(pDataOut)
$define SPI1_DataRdy() DataRdy_SPI1()
$define SPI1_EnableInt() EnableInt_SPI1()
$define SPI1_DisableInt() DisableInt_SPI1()
$define SPI1_SetPriorityInt(pPriority) SetPriorityInt_SPI1(pPriority)

$define SPI_Open(pConfig1,pConfig2) Open_SPI1(pConfig1,pConfig2)
$define SPI_ConfigInt(pConfig) ConfigInt_SPI1(pConfig)
$define SPI_Close() Close_SPI1()
$define SPI_Read() Read_SPI1()
$define SPI_Write(pDataOut) Write_SPI1(pDataOut)
$define SPI_DataRdy() DataRdy_SPI1()
$define SPI_EnableInt() EnableInt_SPI1()
$define SPI_DisableInt() DisableInt_SPI1()
$define SPI_SetPriorityInt(pPriority) SetPriorityInt_SPI1(pPriority)
$define SPI_Clear_Intr_Status_Bit() SPI1_Clear_Intr_Status_Bit()
$define SPI_Clear_Recv_OV() SPI1_Clear_Recv_OV()
$define SPI_Tx_Buf_Full() SPI1_Tx_Buf_Full()
$define SPI_Rx_Buf_Full() SPI1_Rx_Buf_Full()

$endif ' (_SPI1CON1)

__SPI__Main:

$endif ' (_SPI33_INC_)
